
/*

  nssl.c: home-made netcat with SSL support

 */


#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <signal.h>

#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "sock.h"
#include "sslut.h"
#include "utils.h"


extern	int	pid;


void	usage		(char *);
void    signexit	(int);
void    signchld	(int);


unsigned char			f_autist = 1, f_hexdump = 0, f_ssl = 1, 
				f_banner = 0, f_listen  = 0, f_command = 0, f_proxy = 0;

static struct termios		term_old, term;
static conxion			cnx;
static struct fdpipe		*fdp;



void	usage (char *arg)
{
	fprintf (stderr, "\n%s [options] <dest ip> <dest port>\n\n", arg);
	fprintf (stderr, "  where [options] are:\n\n"); 
        fprintf (stderr, "  misc  : [-v] to unset autist mode\n");
	fprintf (stderr, "          [-o] to output traffic in hex dump on stderr\n");
	fprintf (stderr, "          [-r] to not use SSL mode (clear text)\n");
	fprintf (stderr, "          [-c] to unset canonical mode for stdin (send data for each keypress)\n");
        fprintf (stderr, "  client: [-b] to read banner and exit (timeout is %d seconds)\n", BANNER_TIMEOUT);
	fprintf (stderr, "          [-p port] to use source port\n");
	fprintf (stderr, "  server: [-l port] to listen on local port (wait for incoming connections)\n");
	fprintf (stderr, "          [-e comm] to execute a command when connection has been established\n");
	fprintf (stderr, "          [-f file] to load a different cert and privkey from PEM file\n");
	fprintf (stderr, "  proxy:  [-x port] to bounce from local port to remote host\n\n");
	exit (1);
}



int	main (int argc, char *argv[])
{
	unsigned int	arg;
	int		sock = 0, sock_proxy = 0, port_proxy = 0;
	char		*pem_file = NULL, *command = NULL;

	tcgetattr (STDIN_FILENO, &term);
	term_old = term;
	term.c_lflag |= ICANON;

	memset (&cnx, '\x0', sizeof (cnx));

	while ( (arg = getopt (argc, argv, "vorcbe:f:l:p:x:")) != -1 ) 
	   {
		switch(arg) 
		   {
			case 'v':
				f_autist   = 0;
				break;
			case 'o':
				f_hexdump  = 1;
				break;
			case 'r':
				f_ssl      = 0;
				break;
			case 'c':
				term.c_lflag &= ~ICANON;
				tcsetattr (STDIN_FILENO, TCSANOW, &term);
				break;
			case 'b':
				f_banner   = 1;
				break;
			case 'e':
				f_command  = 1;
				command    = optarg;
				break;
			case 'f':
				pem_file   = optarg;
				break;
			case 'x':
				f_proxy	   = 1;
				port_proxy = atol (optarg);
				break;
			case 'l':
				f_listen   = 1;
			case 'p':
				cnx.sport  = atol (optarg);
				break;

			default:
			case 'h':
				usage (argv[0]);
		   }
	   }
	
	if ( (argc - optind) == 2 ) 
	   {
		cnx.dip.s_addr = resolve_host_name (argv[optind]);
		cnx.dport = atol (argv[optind+1]);
	   }
	else if (!f_listen)
	   {
		usage (argv[0]);
	   }

	signal (SIGINT,  signexit);
	signal (SIGTERM, signexit);
	signal (SIGCHLD, signchld);

	if (f_proxy)
	   {
		sock_proxy = listen_sock (port_proxy);
                if (sock_proxy < 0) exit (1);
                if (!f_autist)
		   {
			fprintf (stderr, "proxying on port %d to %s:%d...\n", port_proxy, inet_ntoa(cnx.dip), cnx.dport);
		   }
		while (1)
		   {
			sock = wait_for_connect (sock_proxy);
			if (sock == -1 ) exit (1);

			cnx.sock = ssl_connect_ip (&cnx.dip, cnx.dport, cnx.sport, &cnx.ssl_sock, &cnx.ssl_ctx);
			if (cnx.sock < 0) exit (1);

			if (!f_autist)
			   {
				fprintf (stderr, "connected to %s:%d ", inet_ntoa (cnx.dip), cnx.dport);
				if (cnx.sport) fprintf (stderr, "(using source port %d)", cnx.sport);
				fprintf (stderr, "\n");
				ssl_print_cipher (cnx.ssl_sock);
			   }

			ssl_multiplex_loop (cnx.ssl_sock, sock, sock, cnx.sock);
			if (!f_autist) fprintf (stderr, "remote end closed connection\n");

			ssl_close_all (cnx.sock, cnx.ssl_sock, cnx.ssl_ctx);
			close (sock);
 		   }
		exit (1);
	   }

	if (!f_ssl)
	   {
		if (!f_listen)
		   {
			cnx.sock = connect_tcp (&cnx.dip, cnx.dport, cnx.sport);
			if (cnx.sock < 0) exit (1);
			if (!f_autist)
			   {
				fprintf (stderr, "connected to %s:%d ", inet_ntoa (cnx.dip), cnx.dport);
				if (cnx.sport) fprintf (stderr, "(using source port %d)", cnx.sport);
				fprintf (stderr, "\n");
			   }
			if (f_banner)
			   {
				read_banner (cnx.sock);
			   }
			else
			   {
				multiplex_loop (STDIN_FILENO, STDOUT_FILENO, cnx.sock);
				if (!f_autist) fprintf (stderr, "remote end closed connection\n");
			   }
			exit (1);
		   }
		else
		   {
			cnx.sock = listen_sock (cnx.sport);
			if (cnx.sock < 0) exit (1);
			if (!f_autist) fprintf (stderr, "listening on port %d...\n", cnx.sport);
			sock = wait_for_connect (cnx.sock);
			if (sock == -1 ) exit (1);
			if (f_command)
			   {
				fdp = x_popen (command);
				multiplex_loop (fdp->pip_read, fdp->pip_write, sock);
				x_pclose ();
				exit (1);
			   }
			else
			   {
				multiplex_loop (STDIN_FILENO, STDOUT_FILENO, sock);
				if (!f_autist) fprintf (stderr, "remote end closed connection\n");
			   }
			exit (1);
		   }
	   }

	if (!f_listen)
	   {
		cnx.sock = ssl_connect_ip (&cnx.dip, cnx.dport, cnx.sport, &cnx.ssl_sock, &cnx.ssl_ctx);
		if (cnx.sock < 0) exit (1);
		if (!f_autist)
		   {
			fprintf (stderr, "connected to %s:%d ", inet_ntoa (cnx.dip), cnx.dport);
			if (cnx.sport) fprintf (stderr, "(using source port %d)", cnx.sport);
			fprintf (stderr, "\n");
			ssl_print_cipher (cnx.ssl_sock);
		   }
		if (f_banner)
		   {
			ssl_read_banner (cnx.ssl_sock, cnx.sock);
		   }
		else
		   {
			ssl_multiplex_loop (cnx.ssl_sock, STDIN_FILENO, STDOUT_FILENO, cnx.sock);
			if (!f_autist) fprintf (stderr, "remote end closed connection\n");
		   }
		exit (1);
	   }
	else
	   {
		if (ssl_init_accept (&cnx.ssl_ctx, pem_file) < 0) exit (1);
		if ((cnx.sock = listen_sock (cnx.sport)) < 0 ) exit (1);
		if (!f_autist) fprintf (stderr, "listening on port %d...\n", cnx.sport);
		sock = ssl_wait_for_connection (cnx.sock, &cnx.ssl_sock, cnx.ssl_ctx);
		if (sock == -1)	exit (1);
		if (!f_autist) ssl_print_cipher (cnx.ssl_sock);
		if (f_command)
		   {
			fdp = x_popen (command);
			ssl_multiplex_loop (cnx.ssl_sock, fdp->pip_read, fdp->pip_write, sock);
			x_pclose ();
			exit (1);
		   }
		   else
		   {
			ssl_multiplex_loop (cnx.ssl_sock, STDIN_FILENO, STDOUT_FILENO, sock);
			if (!f_autist) fprintf (stderr, "remote end closed connection\n");
		   }
		exit (1);
	   }

	return 0;
}



void	signexit (int signal)
{
	fprintf (stderr, "closing connection\n");
	tcsetattr (STDIN_FILENO, TCSANOW, &term_old);
	exit (1);
}



void	signchld (int signal)
{
	fprintf (stderr, "child exited\n");
	pid = 0;
}


