

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "nssl.h"
#include "utils.h" 
#include "sock.h"



unsigned long int resolve_host_name (char *hname)
{
	unsigned long	inetaddr;
	struct  hostent *h_ent;

	if ( (inetaddr = inet_addr (hname)) == INADDR_NONE )
	   {
		if ( !(h_ent = gethostbyname (hname)) ) 
		   {
			fprintf (stderr, "can't resolve host %s\n", hname);
			exit (1);
		   }

		bcopy (h_ent->h_addr, (char *)&inetaddr, h_ent->h_length);
	   }

	return (inetaddr);
}



int	connect_tcp (struct in_addr *ip, unsigned short port, unsigned short sport)
{
	int			sock;
	struct sockaddr_in	s_in;

	if (sport)
	   {
		if ( (sock = bind_sock (sport)) < 0 )
		   {
			return (-1);
		   }
	   }
	else
	   {
		if ( (sock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 ) 
		   {
			perror ("socket");
			return (-1);
		   }
	   }

	s_in.sin_family		= AF_INET;
	s_in.sin_port		= htons(port);
        s_in.sin_addr.s_addr	= ip->s_addr;
 
	if ( connect (sock, (struct sockaddr *) &s_in, sizeof(s_in)) < 0 ) 
	   {
		perror ("connect");
		return (-1);
	   }

	return (sock);
}



int	set_reuseaddr (int sock, int val)
{
	if ( setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof (val)) == -1)
	   {
		perror ("setsockopt");
		return (-1);
	   }

	return (0);
}



int     listen_sock (unsigned short int port)
{
	int	sock;

	if ( (sock = bind_sock (port)) < 0 ) 
	   {
		return (-1);
	   }

	if ( listen (sock, 5) < 0 ) 
	   {
		perror ("listen");
		return (-1);
	   }
 
	return (sock);
}



int     bind_sock (unsigned short int port)
{
	int			sock;
	struct sockaddr_in      sin;

	if ( (sock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 ) 
	   {
		perror ("socket");
		return (-1);
	   }

	set_reuseaddr (sock, 1);

	sin.sin_family		= AF_INET;
	sin.sin_port		= htons(port);
        sin.sin_addr.s_addr	= INADDR_ANY;
 
	if ( bind (sock, (struct sockaddr *) &sin, sizeof(sin)) < 0 ) 
	   {
		perror ("bind");
		return (-1);
	   }

	return (sock);
}



int	wait_for_connect (int sock)
{
	int			fd, len = sizeof (struct sockaddr_in);
	struct sockaddr_in	sin;
	
        if ( (fd = accept (sock, (struct sockaddr *) &sin, &len)) == -1 )
	   {
		perror ("accept");
		return (-1);
	   }

	if (!f_autist) fprintf (stderr, "connection received from %s port %d\n", 
			                inet_ntoa (sin.sin_addr), ntohs(sin.sin_port));

	return (fd);
}



int     multiplex_loop (int sockin, int sockprint, int sockout)
{
        int                     num = 0, fdhigh, inclose = 0;
        unsigned char           buf[4096];
        fd_set                  fdread;

        while (1)
           {

                FD_ZERO (&fdread);
                FD_SET  (sockout, &fdread);
                if (!inclose) 
		   {
			FD_SET  (sockin,  &fdread);
			fdhigh = (sockin <= sockout) ? sockout : sockin;
		   }
		else
		   {
			fdhigh = sockout;
		   }

                select (fdhigh+1, &fdread, NULL, NULL, NULL);

                if ( FD_ISSET (sockout, &fdread) )
                   {
                        num = read (sockout, buf, sizeof (buf));
                        if (num <= 0) return -1;
                        if (f_hexdump) hexdump (DATA_INPUT, buf, num);
                        num = write (sockprint, buf, num);
                        if (num <= 0) return -1;
                   }

                if ( FD_ISSET (sockin, &fdread) )
                   {
                        num = read (sockin, buf, sizeof (buf));
                        if (num <= 0)
                           {
				if (sockin == STDIN_FILENO)
				   {
					inclose = 1;
				   }
				else
				   {
					return -1;
				   }
			   }
                        else
                           {
                                if (f_hexdump) hexdump (DATA_OUTPUT, buf, num);
                                num = write (sockout, buf, num);
                                if (num <= 0) return -1;
                           }
                   }
           }

        return 0;
}



int	data_available (int fd, int sec)
{
	struct timeval	timeout;
	fd_set		fds;

	timeout.tv_sec  = sec;
	timeout.tv_usec = 0;

	FD_ZERO (&fds);
	FD_SET  (fd, &fds);

	if (select (fd+1, &fds, NULL, NULL, &timeout) == -1)
	   {
		perror ("select");
		return (-1);
	   }

	if (!FD_ISSET (fd, &fds))
	   {
		fprintf (stderr,"select timeout\n");
		return (-1);
	   }

	return (1);
}



int	set_nonblocking_mode (int fd)
{
	int sockflag;

	sockflag = fcntl (fd, F_GETFL, 0);

	if (fcntl (fd, F_SETFL, sockflag | O_NONBLOCK) == -1)
	   {
		fprintf(stderr, "could not set O_NONBLOCK on socket\n");
		return (-1);
	   }

	return (sockflag);
}



int	unset_nonblocking_mode (int fd, int oldflags)
{
	if (fcntl (fd, F_SETFL, oldflags&(~O_NONBLOCK)) == -1)
	   {
		fprintf(stderr, "could not unset O_NONBLOCK on socket\n");
		return (-1);
	   }

	return (0);
}



int	read_banner (int socket)
{
	int	num;
	char	buf[2048];

	if (data_available (socket, BANNER_TIMEOUT) < 0)
	   {
		return (-1);
	   }

	num = read (socket, buf, sizeof (buf));
	
	if (num > 0)
	   {
                if (f_hexdump) hexdump (DATA_INPUT, buf, num);
                write (STDOUT_FILENO, buf, num);
		return (0);
	   }
	else
	   {
		return (-1);
	   }
}


